/*
 * Copyright © 2021-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel24-VierConnects-core.
 *
 * ipspiel24-VierConnects-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel24-VierConnects-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel24-VierConnects-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel24.VierConnects.core.domain;

/**
 * Represents a direction on the Vier Connects board.
 */
public enum VierConnectsDirection {

    /**
     * Upwards.
     */
    NORTH {
        @Override
        public VierConnectsPosition step(final VierConnectsPosition origin) {
            return origin.offset(-1, 0);
        }
    },

    /**
     * Upwards and to the right.
     */
    NORTHEAST {
        @Override
        public VierConnectsPosition step(final VierConnectsPosition origin) {
            return origin.offset(-1, 1);
        }
    },

    /**
     * To the right.
     */
    EAST {
        @Override
        public VierConnectsPosition step(final VierConnectsPosition origin) {
            return origin.offset(0, 1);
        }
    },

    /**
     * Downwards and to the right.
     */
    SOUTHEAST {
        @Override
        public VierConnectsPosition step(final VierConnectsPosition origin) {
            return origin.offset(1, 1);
        }
    },

    /**
     * Downwards.
     */
    SOUTH {
        @Override
        public VierConnectsPosition step(final VierConnectsPosition origin) {
            return origin.offset(1, 0);
        }
    },

    /**
     * Downwards and to the left.
     */
    SOUTHWEST {
        @Override
        public VierConnectsPosition step(final VierConnectsPosition origin) {
            return origin.offset(1, -1);
        }
    },

    /**
     * To the left.
     */
    WEST {
        @Override
        public VierConnectsPosition step(final VierConnectsPosition origin) {
            return origin.offset(0, -1);
        }
    },

    /**
     * Upwards and to the left.
     */
    NORTHWEST {
        @Override
        public VierConnectsPosition step(final VierConnectsPosition origin) {
            return origin.offset(-1, -1);
        }
    };

    /**
     * Performs a step into this direction given an origin. Note that the resulting position is not checked against any
     * bounds, so this has to be done by the caller if necessary.
     *
     * @param origin The origin.
     * @return The resulting position.
     */
    public abstract VierConnectsPosition step(VierConnectsPosition origin);
}
