/*
 * Copyright © 2021-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel24-VierConnects-core.
 *
 * ipspiel24-VierConnects-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel24-VierConnects-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel24-VierConnects-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel24.VierConnects.core.domain;

import de.fhdw.gaming.core.domain.GameException;

/**
 * A builder which allows to create a Vier Connects player.
 */
public interface VierConnectsPlayerBuilder {

    /**
     * Changes the name of the player.
     * <p>
     * There is no default.
     *
     * @param newName The name of the player.
     * @return {@code this}
     */
    VierConnectsPlayerBuilder changeName(String newName);

    /**
     * Determines whether the player is using crosses or noughts.
     * <p>
     * The default value if not called is {@code true}.
     *
     * @param newUsingCrosses If {@code true}, the player will be using crosses, else she will be using noughts.
     * @return {@code this}
     */
    VierConnectsPlayerBuilder changeUsingCrosses(boolean newUsingCrosses);

    /**
     * Returns {@code true} if this player builder uses crosses, and {@code false} if she uses noughts.
     */
    boolean isUsingCrosses();

    /**
     * Builds the player.
     *
     * @return The Vier Connects player.
     * @throws GameException if creating the player is not allowed by the rules of the game.
     */
    VierConnectsPlayer build() throws GameException;
}
