/*
 * Copyright © 2021-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel24-VierConnects-core.
 *
 * ipspiel24-VierConnects-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel24-VierConnects-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel24-VierConnects-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel24.VierConnects.core.domain.impl;

import java.util.Optional;

import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsPlayer;
import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsPlayerBuilder;

/**
 * Implements {@link VierConnectsPlayerBuilder}.
 */
final class VierConnectsPlayerBuilderImpl implements VierConnectsPlayerBuilder {

    /**
     * The name of the player.
     */
    private Optional<String> name;

    /**
     * {@code true} if the player uses crosses, or {@code false} if she uses noughts.
     */
    private boolean usingCrosses;

    /**
     * Creates an {@link VierConnectsPlayerBuilderImpl}.
     */
    VierConnectsPlayerBuilderImpl() {
        this.name = Optional.empty();
        this.usingCrosses = true;
    }

    @Override
    public VierConnectsPlayerBuilderImpl changeName(final String newName) {
        this.name = Optional.of(newName);
        return this;
    }

    @Override
    public VierConnectsPlayerBuilderImpl changeUsingCrosses(final boolean newUsingCrosses) {
        this.usingCrosses = newUsingCrosses;
        return this;
    }

    @Override
    public boolean isUsingCrosses() {
        return this.usingCrosses;
    }

    @Override
    public VierConnectsPlayer build() throws GameException {
        return new VierConnectsPlayerImpl(this.name.orElseThrow(), this.usingCrosses);
    }
}
